@@ -31,9 +31,7 @@
 
 #include <glib/gstdio.h>
 
-#ifdef HAVE_FRIBIDI
-#include <fribidi/fribidi.h>
-#endif
+#include "mini-fribidi/fribidi_types.h"
 
 #ifndef HAVE_FLOCKFILE
 #  define flockfile(f) (void)1
@@ -1015,8 +1013,8 @@ pango_matrix_get_type (void)
  * 
  * Copies a #PangoMatrix.
  * 
- * Return value: a copy of @matrix. The result must be freed with
- *  pango_matrix_free().
+ * Return value: the newly allocated #PangoMatrix, which should
+ *               be freed with pango_matrix_free().
  *
  * Since: 1.6
  **/
@@ -1156,6 +1154,61 @@ pango_matrix_concat (PangoMatrix       *matrix,
   matrix->y0  = tmp.yx * new_matrix->y0 + tmp.yy * new_matrix->y0 + tmp.y0;
 }
 
+/**
+ * pango_matrix_get_font_scale_factor:
+ * @matrix: a #PangoMatrix, may be %NULL
+ * 
+ * Returns the scale factor of a matrix on the height of the font.
+ * That is, the scale factor in the direction perpendicular to the
+ * vector that the X coordinate is mapped to.
+ *
+ * Return value: the scale factor of @matrix on the height of the font,
+ * or 1.0 if @matrix is %NULL.
+ *
+ * Since: 1.12
+ **/
+double
+pango_matrix_get_font_scale_factor (const PangoMatrix *matrix)
+{
+/*
+ * Based on cairo-matrix.c:_cairo_matrix_compute_scale_factors()
+ *
+ * Copyright 2005, Keith Packard
+ */
+  double det;
+  
+  if (!matrix)
+    return 1.0;
+  
+  det = matrix->xx * matrix->yy - matrix->yx * matrix->xy;
+
+  if (det == 0)
+    {
+      return 0.0;
+    }
+  else
+    {
+      double x = matrix->xx;
+      double y = matrix->yx;
+      double major, minor;
+
+      major = sqrt (x*x + y*y);
+      
+      /*
+       * ignore mirroring
+       */
+      if (det < 0)
+	det = - det;
+      
+      if (major)
+	minor = det / major;
+      else 
+	minor = 0.0;
+
+      return minor;
+    }
+}
+
 static const char canon_map[256] = {
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
    0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
@@ -1286,7 +1339,7 @@ pango_language_from_string (const char *language)
  * or the range is a prefix of the tag, and the character after the
  * tag is '-'.
  *
- * Returns: %TRUE if a match was found.
+ * Return value: %TRUE if a match was found.
  **/
 gboolean
 pango_language_matches (PangoLanguage *language,
@@ -1717,6 +1770,30 @@ pango_find_base_dir (const gchar *text,
   return dir;
 }
 
+/**
+ * pango_unichar_direction:
+ * @ch: a Unicode character
+ *
+ * Determines the direction of a character; either
+ * %PANGO_DIRECTION_LTR, %PANGO_DIRECTION_RTL, or
+ * %PANGO_DIRECTION_NEUTRAL.
+ *
+ * Return value: the direction of the character, as used in the
+ * Unicode bidirectional algorithm.
+ */
+PangoDirection
+pango_unichar_direction (gunichar ch)
+{
+  FriBidiCharType fribidi_ch_type = _pango_fribidi_get_type (ch);
+
+  if (!FRIBIDI_IS_LETTER (fribidi_ch_type))
+    return PANGO_DIRECTION_NEUTRAL;
+  else if (FRIBIDI_IS_RTL (fribidi_ch_type))
+    return PANGO_DIRECTION_RTL;
+  else
+    return PANGO_DIRECTION_LTR;
+}
+
 /**
  * pango_is_zero_width:
  * @ch: a Unicode character
