@@ -30,14 +30,6 @@
 
 #include FT_TRUETYPE_TABLES_H
 
-typedef struct _PangoFcMetricsInfo  PangoFcMetricsInfo;
-
-struct _PangoFcMetricsInfo
-{
-  const char       *sample_str;
-  PangoFontMetrics *metrics;
-};
-
 enum {
   PROP_0,
   PROP_PATTERN
@@ -402,6 +394,40 @@ max_glyph_width (PangoLayout *layout)
   return max_width;
 }
 
+PangoFontMetrics *
+pango_fc_font_create_metrics_for_context (PangoFcFont   *fcfont,
+					  PangoContext  *context)
+{
+  PangoFontMetrics *metrics;
+  PangoLayout *layout;
+  PangoRectangle extents;
+  PangoLanguage *language = pango_context_get_language (context);
+  const char *sample_str = pango_language_get_sample_string (language);
+  
+  metrics = pango_font_metrics_new ();
+  
+  get_face_metrics (fcfont, metrics);
+  
+  layout = pango_layout_new (context);
+  pango_layout_set_font_description (layout, fcfont->description);
+  
+  pango_layout_set_text (layout, sample_str, -1);      
+  pango_layout_get_extents (layout, NULL, &extents);
+  
+  metrics->approximate_char_width = 
+    extents.width / g_utf8_strlen (sample_str, -1);
+
+  pango_layout_set_text (layout, "0123456789", -1);
+  metrics->approximate_digit_width = max_glyph_width (layout);
+
+  g_object_unref (layout);
+
+  return metrics;
+}		    
+
+/* This function is cut-and-pasted into pangocairo-fcfont.c - it might be
+ * better to add a virtual fcfont->create_context (font).
+ */
 static PangoFontMetrics *
 pango_fc_font_get_metrics (PangoFont     *font,
                            PangoLanguage *language)
@@ -425,39 +451,24 @@ pango_fc_font_get_metrics (PangoFont     *font,
 
   if (!tmp_list)
     {
-      PangoLayout *layout;
-      PangoRectangle extents;
       PangoContext *context;
 
+      if (!fcfont->fontmap)
+	return pango_font_metrics_new ();
+
       info = g_new0 (PangoFcMetricsInfo, 1);
+      
       fcfont->metrics_by_lang = g_slist_prepend (fcfont->metrics_by_lang, 
 						 info);
+	
+      info->sample_str = sample_str;
 
-      if (fcfont->fontmap)
-	{
-	  info->sample_str = sample_str;
-	  info->metrics = pango_font_metrics_new ();
-	  
-	  get_face_metrics (fcfont, info->metrics);
+      context = pango_fc_font_map_create_context (PANGO_FC_FONT_MAP (fcfont->fontmap));
+      pango_context_set_language (context, language);
 
-	  context = pango_fc_font_map_create_context (PANGO_FC_FONT_MAP (fcfont->fontmap));
-	  pango_context_set_language (context, language);
-	  
-	  layout = pango_layout_new (context);
-	  pango_layout_set_font_description (layout, fcfont->description);
+      info->metrics = pango_fc_font_create_metrics_for_context (fcfont, context);
 
-	  pango_layout_set_text (layout, sample_str, -1);      
-	  pango_layout_get_extents (layout, NULL, &extents);
-      
-	  info->metrics->approximate_char_width = 
-	    extents.width / g_utf8_strlen (sample_str, -1);
-
-          pango_layout_set_text (layout, "0123456789", -1);
-          info->metrics->approximate_digit_width = max_glyph_width (layout);
-
-	  g_object_unref (layout);
-	  g_object_unref (context);
-	}
+      g_object_unref (context);
     }
 
   return pango_font_metrics_ref (info->metrics);
@@ -510,7 +521,7 @@ pango_fc_font_real_get_glyph (PangoFcFont *font,
  * This face will be kept around until you call
  * pango_fc_font_unlock_face().
  *
- * Returns: the FreeType FT_Face associated with @font.
+ * Return value: the FreeType FT_Face associated with @font.
  *
  * Since: 1.4
  **/
@@ -571,14 +582,14 @@ pango_fc_font_has_char (PangoFcFont *font,
 /**
  * pango_fc_font_get_glyph:
  * @font: a #PangoFcFont
- * @wc: Unicode codepoint to look up
+ * @wc: Unicode character to look up
  * 
- * Gets the glyph index for a given unicode codepoint
+ * Gets the glyph index for a given Unicode character
  * for @font. If you only want to determine
  * whether the font has the glyph, use pango_fc_font_has_char().
  * 
- * Return value: the glyph index, or 0, if the unicode
- *  codepoint doesn't exist in the font.
+ * Return value: the glyph index, or 0, if the Unicode
+ *   character doesn't exist in the font.
  *
  * Since: 1.4
  **/
@@ -686,7 +697,7 @@ pango_fc_font_kern_glyphs (PangoFcFont      *font,
  *
  * This will return any custom decoder set on this font.
  *
- * Returns: The custom decoder
+ * Return value: The custom decoder
  *
  * Since: 1.6
  **/
@@ -759,6 +770,8 @@ get_per_char (FT_Face      face,
  * The other possibility would be to to make it public in something
  * like it's current form, and also expose glyph information
  * caching functionality similar to pango_ft2_font_set_glyph_info().
+ *
+ * Since: 1.6
  **/
 void
 pango_fc_font_get_raw_extents (PangoFcFont    *fcfont,
