@@ -30,7 +30,6 @@
 #include "pango-utils.h"
 #include "pangowin32.h"
 #include "pangowin32-private.h"
-#include "modules.h"
 
 #define PANGO_WIN32_UNKNOWN_FLAG 0x10000000
 
@@ -43,14 +42,6 @@ HDC pango_win32_hdc;
 OSVERSIONINFO pango_win32_os_version_info;
 gboolean pango_win32_debug = FALSE;
 
-typedef struct _PangoWin32MetricsInfo PangoWin32MetricsInfo;
-
-struct _PangoWin32MetricsInfo
-{
-  const char *sample_str;
-  PangoFontMetrics *metrics;
-};
-
 static void pango_win32_font_dispose    (GObject             *object);
 static void pango_win32_font_finalize   (GObject             *object);
 
@@ -132,16 +123,6 @@ PangoContext *
 pango_win32_get_context (void)
 {
   PangoContext *result;
-  static gboolean registered_modules = FALSE;
-  int i;
-
-  if (!registered_modules)
-    {
-      registered_modules = TRUE;
-      
-      for (i = 0; _pango_included_win32_modules[i].list; i++)
-        pango_module_register (&_pango_included_win32_modules[i]);
-    }
 
   result = pango_context_new ();
   pango_context_set_font_map (result, pango_win32_font_map_for_display ());
@@ -198,7 +179,7 @@ pango_win32_get_dc (void)
  *
  * Returns wether debugging is turned on.
  * 
- * Returns: %TRUE if debugging is turned on.
+ * Return value: %TRUE if debugging is turned on.
  * 
  * Since: 1.2
  */
@@ -414,6 +395,24 @@ pango_win32_render (HDC               hdc,
   g_free (dX);
 }
 
+/**
+ * pango_win32_render_transformed:
+ * @hdc:     a windows device context
+ * @matrix:  a #PangoMatrix, or %NULL to use an identity transformation
+ * @font:    the font in which to draw the string
+ * @glyphs:  the glyph string to draw
+ * @x:       the x position of the start of the string (in Pango
+ *           units in user space coordinates)
+ * @y:       the y position of the baseline (in Pango units
+ *           in user space coordinates)
+ *
+ * Renders a #PangoGlyphString onto a windows DC, possibly
+ * transforming the layed-out coordinates through a transformation
+ * matrix. Note that the transformation matrix for @font is not
+ * changed, so to produce correct rendering results, the @font
+ * must have been loaded using a #PangoContext with an identical
+ * transformation matrix to that passed in to this function.
+ **/
 void 
 pango_win32_render_transformed (HDC                hdc,
 				const PangoMatrix *matrix,
@@ -697,8 +696,7 @@ pango_win32_font_logfont (PangoFont *font)
 /**
  * pango_win32_font_select_font:
  * @font: a #PangoFont from the Win32 backend
- * @scale: location to store a scale factor from Windows
- *    units to Pango Units.
+ * @hdc: a windows device context
  * 
  * Selects the font into the specified DC and changes the mapping mode
  * and world transformation of the DC appropriately for the font.
@@ -706,7 +704,9 @@ pango_win32_font_logfont (PangoFont *font)
  * to SaveDC and RestoreDC. Call pango_win32_font_done_font() when
  * you are done using the DC to release allocated resources.
  *
- * See 
+ * See pango_win32_font_get_metrics_factor() for information about
+ * converting from the coordinate space used by this function
+ * into Pango units.
  * 
  * Return value: %TRUE if the operation succeeded.
  **/
@@ -874,12 +874,14 @@ pango_win32_get_unknown_glyph (PangoFont *font,
 
 /**
  * pango_win32_render_layout_line:
- * @hdc:       HDC to use for uncolored drawing
+ * @hdc:       DC to use for uncolored drawing
  * @line:      a #PangoLayoutLine
  * @x:         the x position of start of string (in pixels)
  * @y:         the y position of baseline (in pixels)
  *
- * Render a #PangoLayoutLine onto a device context
+ * Render a #PangoLayoutLine onto a device context. For underlining to
+ * work property the text alignment of the DC should have TA_BASELINE
+ * and TA_LEFT.
  */
 void 
 pango_win32_render_layout_line (HDC              hdc,
@@ -953,11 +955,15 @@ pango_win32_render_layout_line (HDC              hdc,
 	  points[0].y = points[1].y = y + 4;
 	  points[1].x = x + PANGO_PIXELS (x_off + ink_rect.x + ink_rect.width);
 	  Polyline (hdc, points, 2);
-	  /* Fall through */
-	case PANGO_UNDERLINE_SINGLE:
 	  points[0].y = points[1].y = y + 2;
 	  Polyline (hdc, points, 2);
 	  break;
+	case PANGO_UNDERLINE_SINGLE:
+	  points[0].x = x + PANGO_PIXELS (x_off + ink_rect.x) - 1;
+	  points[0].y = points[1].y = y + 2;
+	  points[1].x = x + PANGO_PIXELS (x_off + ink_rect.x + ink_rect.width);
+	  Polyline (hdc, points, 2);
+	  break;
 	case PANGO_UNDERLINE_ERROR:
           {
             int point_x;
@@ -1363,7 +1369,7 @@ font_get_cmap (PangoFont *font)
  *
  * Obtains the index of the glyph for @wc in @font.
  *
- * Returns: the glyph index for @wc.
+ * Return value: the glyph index for @wc.
  **/
 gint
 pango_win32_font_get_glyph_index (PangoFont *font,
