@@ -96,7 +96,7 @@ struct _PangoXSizeInfo
   GSList *xlfds;
 };
 
-const struct {
+static const struct {
   const gchar *text;
   PangoWeight value;
 } weights_map[] = {
@@ -113,7 +113,7 @@ const struct {
   { "black",     900 }
 };
 
-const struct {
+static const struct {
   const gchar *text;
   PangoStyle value;
 } styles_map[] = {
@@ -122,7 +122,7 @@ const struct {
   { "o", PANGO_STYLE_OBLIQUE }
 };
 
-const struct {
+static const struct {
   const gchar *text;
   PangoStretch value;
 } stretches_map[] = {
@@ -266,7 +266,7 @@ static GList *fontmaps = NULL;
  * not be freed. If the font map for a display is no longer needed, it can 
  * be released with pango_x_shutdown_display().
  *
- * Returns: a #PangoXFontMap for @display.
+ * Return value: a #PangoXFontMap for @display.
  **/
 PangoFontMap *
 pango_x_font_map_for_display (Display *display)
@@ -1334,7 +1334,7 @@ pango_x_make_matching_xlfd (PangoFontMap *fontmap, char *xlfd, const char *chars
  *
  * Obtains the font cache associated with the given font map.
  *
- * Returns: the #PangoXFontCache of @font_map.
+ * Return value: the #PangoXFontCache of @font_map.
  **/
 PangoXFontCache *
 pango_x_font_map_get_font_cache (PangoFontMap *font_map)
@@ -1512,6 +1512,22 @@ pango_x_face_get_type (void)
   return object_type;
 }
 
+/* Cut and paste here to avoid an inter-module dependency */
+static PangoCoverageLevel
+engine_shape_covers (PangoEngineShape *engine,
+		     PangoFont        *font,
+		     PangoLanguage    *language,
+		     gunichar          wc)
+{
+  g_return_val_if_fail (PANGO_IS_ENGINE_SHAPE (engine), PANGO_COVERAGE_NONE);
+  g_return_val_if_fail (PANGO_IS_FONT (font), PANGO_COVERAGE_NONE);
+
+  return PANGO_ENGINE_SHAPE_GET_CLASS (engine)->covers (engine,
+							font,
+							language,
+							wc);
+}
+
 PangoCoverage *
 pango_x_face_get_coverage (PangoXFace      *xface,
 			   PangoFont       *font,
@@ -1558,7 +1574,7 @@ pango_x_face_get_coverage (PangoXFace      *xface,
 	{
 	  PangoCoverageLevel level;
 
-	  level = _pango_engine_shape_covers (engine, font, language, wc);
+	  level = engine_shape_covers (engine, font, language, wc);
 	  if (level != PANGO_COVERAGE_NONE)
 	    pango_coverage_set (result, wc, level);
 	}
